# Scripts for the Yiou et al. Clim. Past (2010) paper to
# test the computations of Le Mouel et al. (2009)
# A directory ~/data/CPDwork should be created. It should contain the
# temperature and geomagnetic data.
# DISCLAIMER:
# This script was made for research use only. It was not designed by
# profesional programmers and is provided "as is" for academic research
# in order to verify the computations done by Yiou et al. Clim. Past Discuss
# (2010). The main contributors to the script are P. Yiou and H.W. Rust. Some
# parts (the L and Q estimates for temperature and geomagnetic activity) were
# implemented in Mathematica by B. Legras and give exactly the same
# results (Figs. 4 and 5).
# The data sets are freely available on the web at:
# www.ecad.knmi.nl (temperature data)
# ???.uk (geomagnetic data)
# The user should be able to retrieve him/herself the data and put them
# in the relevant format. We could provide programs in R to do this, but
# such scripts would make the present programme unnessarily long.
# If errors/bugs are found, please report them to Pascal Yiou, who
# will be happy to make the necessary changes.
# Also note that this script does not generate figure in numerical order.
# P. Yiou was too tired to translate all comments into English, but the French
# instructions are rather trivial.

SI=Sys.info()
if(SI[[1]] == "Darwin"){
  # The paths should be adapted to the system:
  # you must specify here where you store R code and datasets
Rsource="~/textes/lifetime/R_programs/"
DATdir="~/textes/lifetime/"
}

source(paste(Rsource,"plotCI.R",sep=""))

# DEFINITIONS OF FUNCTIONS
# Compute p-values for correlations, with a given dof
"pval.cor"=function(correl,ndof)
{
  t.cor=abs(correl)/sqrt(1-correl^2)
  t.cor=sqrt(ndof-2)*t.cor
  p.val=2*(1-pt(t.cor,ndof-2))
  p.val
}

# Compute a seasonal cycle from a time series
"seas.cyc" <- function(XT,time,lyear=c(1961,1990))
{
  X.cyc=c()
  Xanom=XT
  for(month in 1:12){
    rday=range(time$day[time$month==month])
    for(day in rday[1]:rday[2]){
      mX=mean(XT[time$month==month & time$day==day & 
        time$year>=lyear[1] & time$year<=lyear[2]],na.rm=TRUE)
      X.cyc=c(X.cyc,mX)
      Xanom[time$day==day & time$month==month]=
        t(t(XT[time$day==day & time$month==month])-mX)
    }
  }
  list(cycle=X.cyc,anom=Xanom)
}# end seas.cyc

# Generating Gaussian AR(1)
"rednoise" <-function(a,sigmb,N)
{
  RN=list()
  RN=c()
  Bt=rnorm(N,0,sigmb)
  RN=c(RN,Bt[1])
  dum=Bt[1]
  for(i in c(2:N)){
    dum=a*dum+Bt[i]
    RN=c(RN,dum)
  }
  RN
}

# Generating AR(1) with time-varying memory parameter
"trednoise" <- function(a,sigmb,N)
{
n.a=length(a)
if(n.a<N) a=rep(a,N)
RN=c()
Bt=rnorm(N,0,sigmb)
RN=c(RN,Bt[1])
dum=Bt[1]
for(i in c(2:N)){
dum=a[i]*dum+Bt[i]
RN=c(RN,dum)
}
RN
}

### mean square daily/annual variation
# Slow version (with a loop) but more robust to NA
msdv <- function(z,theta=365,delta=1){
N=length(z)
z1= z[c((delta+1):N,1:delta)]

zsqr= rep(NA,N)

theta2= floor(theta/2)
for(t in (theta2+delta):(N-theta2-delta))
zsqr[t] = mean((z1[(t-theta2):(t+theta2)]-z[(t-theta2):(t+theta2)])**2,
                na.rm=TRUE)
return(zsqr)
}

# Slow lifetime estimator (with a loop) but more robust to NA
lifetime <- function(z,delta1=365,delta2=1,theta=11*365){
msdv1=msdv(z,theta,delta1)
msdv2= msdv(z,theta,delta2)
return(msdv1/msdv2)
}

# Fast mean square daily deviations estimate (using filter)
"msdv2" = function(X,Nyear=11,lag=365,Nbday=365)
  {
    NX=length(X)
    Q=rep(NA,NX)
    deltalag=(X[(lag+1):NX]-X[1:(NX-lag)])^2
    Qdum=filter(deltalag,rep(1,Nyear*Nbday)/(Nyear*Nbday))
    Q[(floor(lag/2)+1):(NX-floor(lag/2)-1)]=Qdum
    invisible(Q)
  }

# Fast "lifetime estimator (using filter)
"lifetime2"=function(X,Nyear=11,lag=365,Nbday=365)
  {
    Q=msdv2(X,Nyear,lag,Nbday)
    zeta=msdv2(X,Nyear,lag=1,Nbday)
    L=Q/zeta
    invisible(L)
  }
# END OF PRELIMINARY FUNCTION DEFINITIONS

# Figure 3:
# Testing the bias of $a$ estimators (MLE and LM et al.)
# for AR(1) processes, with different values of $a$
# We use the fast "lifetime" estimator here.
list.a=c()
list.lsta=c()
list.ltru=c()
list.lLM=c()
list.lLM05=c()
list.lLM95=c()
nexp=50
for(i in 1:nexp){
  lambd=20*(i/nexp)
  a=1-1/lambd
  X=rednoise(a,1,30000)
  C.X.0=var(X)
  C.X.1=cov(X[1:(length(X)-1)],X[2:length(X)])
  a.star=C.X.1/C.X.0
  L.X=lifetime2(X)
  lambda.LM=mean(L.X,na.rm=TRUE)
  lambda.LM05=quantile(L.X,probs=0.05,na.rm=TRUE)
  lambda.LM95=quantile(L.X,probs=0.95,na.rm=TRUE)
  lambda.star=1/(1-a.star)
  lambda.true=1/(1-a)
  list.a=c(list.a,a)
  list.lsta=c(list.lsta,lambda.star)
  list.ltru=c(list.ltru,lambda.true)
  list.lLM=c(list.lLM,lambda.LM)
  list.lLM05=c(list.lLM05,lambda.LM05)
  list.lLM95=c(list.lLM95,lambda.LM95)
}

setwd(paste(DATdir,"CPDwork",sep=""))
pdf("bias_AR1.pdf")
plot(c(1,20),c(1,25),type="n",
xlab="True lambda",ylab="Estimated lambda",xlim=c(0,20),ylim=c(0,25))
abline(a=0,b=1)
points(list.ltru,list.lsta,col="red",bg="red",pch=19)
plotCI(list.ltru,list.lLM,list.lLM-list.lLM05,list.lLM95-list.lLM,
       add=TRUE,pch=19,bg="black")
l.a=c(0,0.5,0.75,0.9,0.925,0.95,0.99,0.995)
axis(side=3,at=1/(1-l.a),label=l.a)
mtext(side=3,line=2,text="a")
legend("topleft",legend=c("L(t) estimate","MLE estimate"),pch=19,
       col=c("black","red"),
       pt.bg=c("red","black"))
dev.off()

# Figure 4
# Comparison between LT of TG in Europe and that of an AR(1)
# with same variance and auto-covariance
setwd(paste(DATdir,"CPDwork",sep=""))
# Daily spatial European mean
TGEur=read.table("TGmean_1900.dat")
TGEurano=read.table("TGmean_1900anom.dat")

# Extracting days, months and years
dayEur=TGEur[[1]] %% 100
monthEur=floor(TGEur[[1]]/100) %% 100
yearEur=floor(TGEur[[1]]/10000)
frac.yearEur=yearEur+(monthEur-1)/12+(dayEur-1)/(31*100)

L.TGEur=lifetime2(TGEur[,2]/10) # ECA&D gives 1/10 of K
L.TGEurano=lifetime2(TGEurano[,2]/10)
L.TGEur7=lifetime2(TGEur[,2]/10,Nyear=7)
L.TGEur22=lifetime2(TGEur[,2]/10,Nyear=22)

Q.TGEurano=msdv2(TGEurano[,2]/10)

# variance et autocovariance
C0.TGEur=var(TGEurano[,2]/10)
C1.TGEur=cov(TGEurano[1:(nrow(TGEurano)-1),2]/10,
  TGEurano[2:nrow(TGEurano),2]/10)
a.TGEur=C1.TGEur/C0.TGEur
varB.TGEur=C0.TGEur-a.TGEur*C1.TGEur

# Simulating AR(1) with same variance and autocovariance
X=rednoise(a.TGEur,sqrt(varB.TGEur),nrow(TGEurano)*2)
L.X=lifetime2(X)
Q.X=msdv2(X)
LEur.bounds=quantile(L.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
QEur.bounds=quantile(Q.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
L.X7=lifetime2(X,Nyear=7)
Q.X7=msdv2(X,Nyear=7)
L.X22=lifetime2(X,Nyear=22)
Q.X22=msdv2(X,Nyear=22)
LEur.bounds=quantile(L.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
QEur.X.bounds=quantile(Q.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
LEur7.bounds=quantile(L.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
QEur7.X.bounds=quantile(Q.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
LEur22.bounds=quantile(L.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)
QEur22.X.bounds=quantile(Q.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)

rEur=range(c(L.TGEur,L.TGEur7),na.rm=TRUE)

# Comparison between LT of TG in Paris and that of an AR(1)
# with same variance and auto-covariance
TGparis=read.table("TG_STAID000038.txt",header=TRUE,skip=18,sep=",")
yy.paris=floor(TGparis$DATE/10000)
mm.paris=floor(TGparis$DATE/100) %% 100
dd.paris=TGparis$DATE %% 100

tt.paris=list(year=yy.paris,month=mm.paris,day=dd.paris)
tt.paris.fr=yy.paris+(mm.paris-1)/12+(dd.paris-1)/(31*100)
# Seasonal anomalies
TGparis.anom=seas.cyc(TGparis$TG,tt.paris)

L.TGparis=lifetime2(TGparis$TG/10)
L.TGanoparis=lifetime2(TGparis.anom$anom/10)
Q.TGanoparis=msdv2(TGparis.anom$anom/10)
L.TGparis2=lifetime2(TGparis.anom$anom/10,Nyear=7)
L.TGparis3=lifetime2(TGparis.anom$anom/10,Nyear=15)
L.TGparis4=lifetime2(TGparis.anom$anom/10,Nyear=22)

C0.TGparis=var(TGparis.anom$anom/10)
C1.TGparis=cov(TGparis.anom$anom[1:(length(TGparis.anom$anom)-1)]/10,
  TGparis.anom$anom[2:length(TGparis.anom$anom)]/10)
a.TGparis=C1.TGparis/C0.TGparis
varB.TGparis=C0.TGparis-a.TGparis*C1.TGparis

# Equivalent AR(1) (same variance and autocovariance)
X=rednoise(a.TGparis,sqrt(varB.TGparis),length(TGparis.anom$anom)*2)
L.X=lifetime2(X)
Q.X=msdv2(X)
L.X7=lifetime2(X,Nyear=7)
Q.X7=msdv2(X,Nyear=7)
L.X22=lifetime2(X,Nyear=22)
Q.X22=msdv2(X,Nyear=22)
Lparis.bounds=quantile(L.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qparis.X.bounds=quantile(Q.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Lparis7.bounds=quantile(L.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qparis7.X.bounds=quantile(Q.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Lparis22.bounds=quantile(L.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qparis22.X.bounds=quantile(Q.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)

rparis=range(c(L.TGparis,L.TGanoparis,Lparis.bounds),na.rm=TRUE)

# Comparison between LT of TG in De Bilt and that of an AR(1)
# with same variance and auto-covariance
TGdblt=read.table("TG_STAID000162.txt",header=TRUE,skip=18,sep=",")
yy.dblt=floor(TGdblt$DATE/10000)
mm.dblt=floor(TGdblt$DATE/100) %% 100
dd.dblt=TGdblt$DATE %% 100

tt.dblt=list(year=yy.dblt,month=mm.dblt,day=dd.dblt)
tt.dblt.fr=yy.dblt+(mm.dblt-1)/12+(dd.dblt-1)/(31*100)
# Seasonal anomalies
TGdblt.anom=seas.cyc(TGdblt$TG,tt.dblt)

L.TGdblt=lifetime2(TGdblt$TG/10)
L.TGanodblt=lifetime2(TGdblt.anom$anom/10)
Q.TGanodblt=msdv2(TGdblt.anom$anom/10)
L.TGdblt2=lifetime2(TGdblt.anom$anom/10,Nyear=7)
L.TGdblt3=lifetime2(TGdblt.anom$anom/10,Nyear=15)
L.TGdblt4=lifetime2(TGdblt.anom$anom/10,Nyear=22)

C0.TGdblt=var(TGdblt.anom$anom/10)
C1.TGdblt=cov(TGdblt.anom$anom[1:(length(TGdblt.anom$anom)-1)]/10,
  TGdblt.anom$anom[2:length(TGdblt.anom$anom)]/10)
a.TGdblt=C1.TGdblt/C0.TGdblt
varB.TGdblt=C0.TGdblt-a.TGdblt*C1.TGdblt

# Equivalent AR(1) (same variance and autocovariance)
X=rednoise(a.TGdblt,sqrt(varB.TGdblt),length(TGdblt.anom$anom)*2)
L.X=lifetime2(X)
Q.X=msdv2(X)
L.X7=lifetime2(X,Nyear=7)
Q.X7=msdv2(X,Nyear=7)
L.X22=lifetime2(X,Nyear=22)
Q.X22=msdv2(X,Nyear=22)

Ldblt.bounds=quantile(L.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qdblt.X.bounds=quantile(Q.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Ldblt7.bounds=quantile(L.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qdblt7.X.bounds=quantile(Q.X7,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Ldblt22.bounds=quantile(L.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qdblt22.X.bounds=quantile(Q.X22,probs=c(0.05,0.5,0.95),na.rm=TRUE)

rdblt=range(c(L.TGdblt,L.TGanodblt,Ldblt.bounds),na.rm=TRUE)

"figure4" = function()
  {
    layout(matrix(1:6,2,3))
                                        #Fig for De Bilt
    par(mar=c(3,5,1,1))
                                        # Q(t) in De Bilt
    plot(tt.dblt.fr,Q.TGanodblt,type="n",xlab="",ylab="Q(t)")
    plotCI(1900,Qdblt.X.bounds[2],Qdblt.X.bounds[2]-Qdblt.X.bounds[1],
           Qdblt.X.bounds[3]-Qdblt.X.bounds[2],add=TRUE,col="black")
    lines(tt.dblt.fr,Q.TGanodblt)
    legend("topleft",legend="(a)",bty="n")
    legend("topright",legend="De Bilt",bty="n")
                                        # L(t) in De Bilt
    plot(range(tt.dblt.fr),rdblt,type="n",xlab="Year",ylab="L(t)",ylim=c(4,6))
    plotCI(1905,Ldblt.bounds[2],Ldblt.bounds[2]-Ldblt.bounds[1],
           Ldblt.bounds[3]-Ldblt.bounds[2],add=TRUE,col="black",lwd=2)
    plotCI(1900,Ldblt7.bounds[2],Ldblt7.bounds[2]-Ldblt7.bounds[1],
           Ldblt7.bounds[3]-Ldblt7.bounds[2],add=TRUE,col="blue")
    plotCI(1910,Ldblt22.bounds[2],Ldblt22.bounds[2]-Ldblt22.bounds[1],
           Ldblt22.bounds[3]-Ldblt22.bounds[2],add=TRUE,col="brown")
    lines(tt.dblt.fr,L.TGdblt2,col="blue")
    lines(tt.dblt.fr,L.TGdblt4,col="brown")
    lines(tt.dblt.fr,L.TGdblt,lwd=2)
    ex.cs1 <- expression(paste(Theta,"=7yr"),paste(Theta,"=11yr"),
        paste(Theta,"=22yr"))
    legend("bottomright",legend=ex.cs1,
           col=c("blue","black","brown"),lwd=c(1,2,1),ncol=2,bty="n")
    legend("topleft",legend="(b)",bty="n")
    legend("topright",legend="De Bilt",bty="n")
                                        #Trace pour Paris
    par(mar=c(3,3,1,1))
                                        # Q(t) a Paris
    plot(tt.paris.fr,Q.TGanoparis,type="n",xlab="",ylab="Q(t)")
    plotCI(1900,Qparis.X.bounds[2],Qparis.X.bounds[2]-Qparis.X.bounds[1],
           Qparis.X.bounds[3]-Qparis.X.bounds[2],add=TRUE,col="black")
    lines(tt.paris.fr,Q.TGanoparis)
    legend("topleft",legend="(c)",bty="n")
    legend("topright",legend="Paris",bty="n")
                                        # L(t) a Paris
    plot(range(tt.paris.fr),rparis,type="n",xlab="Year",ylab="L(t)")
    plotCI(1905,Lparis.bounds[2],Lparis.bounds[2]-Lparis.bounds[1],
           Lparis.bounds[3]-Lparis.bounds[2],add=TRUE,col="black",lwd=2)
    plotCI(1900,Lparis7.bounds[2],Lparis7.bounds[2]-Lparis7.bounds[1],
           Lparis7.bounds[3]-Lparis7.bounds[2],add=TRUE,col="blue")
    plotCI(1910,Lparis22.bounds[2],Lparis22.bounds[2]-Lparis22.bounds[1],
           Lparis22.bounds[3]-Lparis22.bounds[2],add=TRUE,col="brown")
    lines(tt.paris.fr,L.TGparis2,col="blue")
    lines(tt.paris.fr,L.TGparis4,col="brown")
    lines(tt.paris.fr,L.TGparis,lwd=2)
    ex.cs1 <- expression(paste(Theta,"=7yr"),paste(Theta,"=11yr"),
        paste(Theta,"=22yr"))
    legend("bottomright",legend=ex.cs1,
           col=c("blue","black","brown"),lwd=c(1,2,1),ncol=2,bty="n")
    legend("topleft",legend="(d)",bty="n")
    legend("topright",legend="Paris",bty="n")
                                        #Trace pour la moyenne europenne
    par(mar=c(3,1,1,5))
                                        # Q(t) en Europe
    plot(range(frac.yearEur),range(Q.TGEurano,QEur.bounds,na.rm=TRUE),
         type="n",xlab="",ylab="Q(t)",axes=FALSE)
    axis(side=1)
    axis(side=4)
    box()
    plotCI(1905,QEur.bounds[2],QEur.bounds[2]-QEur.bounds[1],
           QEur.bounds[3]-QEur.bounds[2],add=TRUE)
    lines(tt.paris.fr,Q.TGEurano)
    legend("topleft",legend="(e)",bty="n")
    legend("topright",legend="Europe",bty="n")
                                        # L(t) en Europe
    plot(range(tt.paris.fr),rEur,type="n",xlab="Year",ylab="L(t)",axes=FALSE)
    axis(side=1)
    axis(side=4)
    box()
    plotCI(1905,LEur.bounds[2],LEur.bounds[2]-LEur.bounds[1],
           LEur.bounds[3]-LEur.bounds[2],add=TRUE)
    lines(frac.yearEur,L.TGEur7,col="blue")
    lines(frac.yearEur,L.TGEur22,col="brown")
    lines(frac.yearEur,L.TGEur,lwd=2)
    plotCI(1905,LEur.bounds[2],LEur.bounds[2]-LEur.bounds[1],
           LEur.bounds[3]-LEur.bounds[2],add=TRUE,col="black",lwd=2)
    plotCI(1900,LEur7.bounds[2],LEur7.bounds[2]-LEur7.bounds[1],
           LEur7.bounds[3]-LEur7.bounds[2],add=TRUE,col="blue")
    plotCI(1910,LEur22.bounds[2],LEur22.bounds[2]-LEur22.bounds[1],
           LEur22.bounds[3]-LEur22.bounds[2],add=TRUE,col="brown")
    legend("topleft",legend="(f)",bty="n")
    legend("topright",legend="Europe",bty="n")
     ex.cs1 <- expression(paste(Theta,"=7yr"),paste(Theta,"=11yr"),
        paste(Theta,"=22yr"))
    legend("bottomright",legend=ex.cs1,
           col=c("blue","black","brown"),lwd=c(1,2,1),ncol=2,bty="n")
 }

pdf("TG_lifetime.pdf",width=10,height=6)
figure4()
dev.off()


# Traitement des donnees geomagnetiques (z.comp)
setwd(paste(DATdir,"CPDwork",sep=""))
load("EskZcomp_py.Rdat")

Z.year=floor(z.comp$date/10000)
Z.month=floor(z.comp$date/100) %% 100
Z.day=z.comp$date %% 100
Z.cdate=Z.year+(Z.month-1)/12+(Z.day-1)/(31*100)

I.OK=which(!is.na(z.comp$mean))
I.tot=1:length(z.comp$mean)
# Lissage par spline pour determiner la partie "basse frequence"
z.smooth=smooth.spline(I.tot[I.OK],z.comp$mean[I.OK],df=20)
# Anomalies par rapport a la basse frequence
z.dum=z.comp$mean[I.OK]-z.smooth$y
z.anom=z.comp$mean
z.anom[I.OK]=z.dum

L.zanom=lifetime(z.anom)
Q.zanom=msdv(z.anom,theta=11*365,delta=365)
zeta.zanom=msdv(z.anom,theta=11*365,delta=1)
L.zanom1940=lifetime(z.anom[Z.year>=1940])

C0.Z=var(z.anom,na.rm=TRUE)
C1.Z=cov(z.anom[1:(length(z.anom)-1)],
  z.anom[2:length(z.anom)],use="pairwise")
a.Z=C1.Z/C0.Z
varB.Z=C0.Z-a.Z*C1.Z

C0.Z1940=var(z.anom[Z.year>=1940],na.rm=TRUE)
C1.Z1940=cov(z.anom[Z.year>=1940][1:(length(z.anom[Z.year>=1940])-1)],
z.anom[Z.year>=1940][2:length(z.anom[Z.year>=1940])],use="pairwise")
a.Z1940=1-1/mean(L.zanom1940,na.rm=TRUE)
varB.Z1940=C0.Z1940-a.Z1940*C1.Z1940

# Bruit rouge equivalent
X=rednoise(a.Z1940,sqrt(varB.Z1940),length(z.comp$mean[Z.year>=1940]))
L.X=lifetime2(X)
Q.X=msdv(X,theta=11*365,delta=1)
Lesk.bounds=quantile(L.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)
Qesk.bounds=quantile(Q.X,probs=c(0.05,0.5,0.95),na.rm=TRUE)

resk=range(c(L.zanom,Lesk.bounds),na.rm=TRUE)
Qdum=(Q.zanom-min(Q.zanom,na.rm=TRUE))/(max(Q.zanom,na.rm=TRUE)-
                            min(Q.zanom,na.rm=TRUE))
Qdum=(Qdum)*(max(zeta.zanom,na.rm=TRUE)-min(zeta.zanom,na.rm=TRUE))+
  min(zeta.zanom,na.rm=TRUE)
lab.Qdum=(seq(200,1200,by=200)-
          min(Q.zanom,na.rm=TRUE))/(max(Q.zanom,na.rm=TRUE)-
                        min(Q.zanom,na.rm=TRUE))
lab.Qdum=(lab.Qdum)*(max(zeta.zanom,na.rm=TRUE)-min(zeta.zanom,na.rm=TRUE))+
  min(zeta.zanom,na.rm=TRUE)

# Figure 2: Geomagnetic data
pdf("Zcomp_data.pdf",width=8,height=6)
layout(matrix(1:2,2,1))
par(mar=c(3,4,1,1))
# Trace des donnees
plot(Z.cdate,z.comp$mean,type="l",xlab="",ylab="Z component")
lines(Z.cdate[I.OK],z.smooth$y,col="red",lwd=2)
legend("topleft",legend="(a)",bty="n")
par(mar=c(4,4,0,1))
plot(Z.cdate,z.anom,type="l",xlab="Year",ylab="Z anomaly")
legend("topleft",legend="(b)",bty="n")
dev.off()

# Figure 5
pdf("Zcomp_Qzeta.pdf",width=10,height=6)
layout(matrix(1:2,2,1))
par(mar=c(3,5,1,5))
# Trace de Q(t) et zeta(t)
xls.y=expression(paste(zeta,"(t)"))
plot(Z.cdate,Qdum,type="l",xlab="",ylab=xls.y,axes=FALSE,col="blue")
axis(side=4,at=lab.Qdum,labels=seq(200,1200,by=200),col="blue",col.axis="blue")
mtext(side=4,text="Q(t)",line=3,col="blue")
axis(side=1)
axis(side=2)
box()
lines(Z.cdate,zeta.zanom,col="black")
abline(h=Qesk.bounds,lty=c(3,2,3),lwd=c(1,2,1),col=c("black","red","black"))
xls.leg=expression(paste(zeta,"(t)"),Q(t))
legend("topright",legend=xls.leg,col=c("black","blue"),lwd=c(1,1),bty="n")
legend("topleft",legend="(a)",bty="n")
# Trace de L(t)
plot(Z.cdate,L.zanom,type="l",xlab="",ylab="L(t)",log="y")
abline(h=Lesk.bounds,lty=c(3,2,3),lwd=c(1,2,1),col=c("black","red","black"))
legend("topleft",legend="(b)",bty="n")
dev.off()

# Calculs supplementaires: correlation entre transformations de bruit rouge AR1
ll=dir()
if("corAR1.Rdata" %in% ll){
  load("corAR1.Rdata")
}else{
a1=a.Z
s1=sqrt(varB.Z)
a2=a.TGEur
s2=varB.TGEur
N=30000
nsim=100
l.C.Q1Q2=c()
l.C.L1L2=c()
l.C.zeta12=c()
l.C.L1zeta2=c()
for(i in 1:nsim){
  X1=rednoise(a1,s1,N)
  X2=rednoise(a2,s2,N)
  Q.X1=msdv2(X1)
  L.X1=lifetime2(X1)
  zeta.X1=msdv2(X1,lag=1)
  Q.X2=msdv2(X2)
  L.X2=lifetime2(X2)
  zeta.X2=msdv2(X2,lag=1)
  C.Q1Q2=cor(Q.X1,Q.X2,use="pairwise")
  C.L1L2=cor(L.X1,L.X2,use="pairwise")
  C.zeta12=cor(zeta.X1,zeta.X2,use="pairwise")
  C.L1.zeta2=cor(L.X1,zeta.X2,use="pairwise")
  l.C.Q1Q2=c(l.C.Q1Q2,C.Q1Q2)
  l.C.L1L2=c(l.C.L1L2,C.L1L2)
  l.C.zeta12=c(l.C.zeta12,C.zeta12)
  l.C.L1zeta2=c(l.C.L1zeta2,C.L1.zeta2)
}
save(file="corAR1.Rdata",l.C.Q1Q2,l.C.L1L2,l.C.zeta12,l.C.L1zeta2,a1,s1,a2,s2)
}


# Calculs de correlations
CorTGparisZ=cor(TGparis[[3]][TGparis[[2]] %in% z.comp$date],
  z.comp$mean[z.comp$date %in% TGparis[[2]]],use="pairwise")
CorTGparisZano=cor(TGparis.anom$anom[TGparis[[2]] %in% z.comp$date],
  z.anom[z.comp$date %in% TGparis[[2]]],use="pairwise")
CorLTGparisQZ=cor(L.TGparis[TGparis[[2]] %in% z.comp$date],
  Q.zanom[z.comp$date %in% TGparis[[2]]],use="pairwise")
CorLTGparisLZ=cor(L.TGparis[TGparis[[2]] %in% z.comp$date],
  L.zanom[z.comp$date %in% TGparis[[2]]],use="pairwise")
CorLTGpariszetaZ=cor(L.TGparis[TGparis[[2]] %in% z.comp$date],
  zeta.zanom[z.comp$date %in% TGparis[[2]]],use="pairwise")
pval.cor(CorLTGpariszetaZ,10)

CorTGparisdblt=cor(TGparis.anom$anom[TGparis[[2]] %in% TGdblt[[2]]],
  TGdblt.anom$anom[TGdblt[[2]] %in% TGparis[[2]]],use="pairwise")
CorQTGparisdblt=cor(Q.TGanoparis[TGparis[[2]] %in% TGdblt[[2]]],
  Q.TGanodblt[TGdblt[[2]] %in% TGparis[[2]]],use="pairwise")
CorLTGparisdblt=cor(L.TGanoparis[TGparis[[2]] %in% TGdblt[[2]]],
  L.TGanodblt[TGdblt[[2]] %in% TGparis[[2]]],use="pairwise")

CorTGdbltZ=cor(TGdblt[[3]][TGdblt[[2]] %in% z.comp$date],
z.comp$mean[z.comp$date %in% TGdblt[[2]]],use="pairwise")
CorTGpdbltZano=cor(TGdblt.anom$anom[TGdblt[[2]] %in% z.comp$date],
z.anom[z.comp$date %in% TGdblt[[2]]],use="pairwise")
CorLTGdbltQZ=cor(L.TGdblt[TGdblt[[2]] %in% z.comp$date],
  Q.zanom[z.comp$date %in% TGdblt[[2]]],use="pairwise")
CorLTGdbltzetaZ=cor(L.TGdblt[TGdblt[[2]] %in% z.comp$date],
  zeta.zanom[z.comp$date %in% TGdblt[[2]]],use="pairwise")
CorLTGdbltLZ=cor(L.TGdblt[TGdblt[[2]] %in% z.comp$date],
  L.zanom[z.comp$date %in% TGdblt[[2]]],use="pairwise")

CorTGEurZ=cor(TGEur[,2][TGEur[,1] %in% z.comp$date],
z.comp$mean[z.comp$date %in% TGEur[,1]],use="pairwise")
CorLTGEurQZ=cor(L.TGEur[TGEur[[1]] %in% z.comp$date],
  Q.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorL7TGEurQZ=cor(L.TGEur7[TGEur[[1]] %in% z.comp$date],
  Q.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorL22TGEurQZ=cor(L.TGEur22[TGEur[[1]] %in% z.comp$date],
Q.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorLTGEurzetaZ=cor(L.TGEur[TGEur[[1]] %in% z.comp$date],
  zeta.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorLTGEur22zetaZ=cor(L.TGEur22[TGEur[[1]] %in% z.comp$date],
zeta.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorLTGEur7zetaZ=cor(L.TGEur7[TGEur[[1]] %in% z.comp$date],
zeta.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
CorLTGEurLZ=cor(L.TGEur[TGEur[[1]] %in% z.comp$date],
  L.zanom[z.comp$date %in% TGEur[[1]]],use="pairwise")
pval.cor(CorLTGEurzetaZ,10)
pval.cor(CorLTGEurQZ,10)

# Figure 1
# Trace des donnees en moyennes annuelles
TGparis.yrmean=tapply(TGparis$TG,yy.paris,mean,na.rm=TRUE)
TGdblt.yrmean=tapply(TGdblt$TG,yy.dblt,mean,na.rm=TRUE)
TGEUR.yrmean=tapply(TGEur[,2],yearEur,mean,na.rm=TRUE)
r.T=range(c(TGparis.yrmean,TGdblt.yrmean,TGEUR.yrmean))/10
r.y=range(c(yy.paris,yy.dblt,yearEur))
pdf("TGmeans.pdf")
par(mar=c(5,4,1,1))
plot(r.y,r.T,type="n",xlab="Years",ylab="T mean")
lines(unique(yy.paris),TGparis.yrmean/10)
lines(unique(yy.dblt),TGdblt.yrmean/10,col="red")
lines(unique(yearEur),TGEUR.yrmean/10,col="blue",lwd=2)
legend("topleft",legend=c("Paris","De Bilt","Europe mean"),lwd=c(1,1,2),col=c("black","red","blue"),bty="n")
dev.off()


# Test avec un AR(1) de coefficient variable force par
# l'activite geomagnetique
aParis=1-1/Lparis.bounds
r.zeta=(zeta.zanom-min(zeta.zanom,na.rm=TRUE))/(max(zeta.zanom,na.rm=TRUE)-min(zeta.zanom,na.rm=TRUE))
r.zeta2LT=r.zeta*(aParis[3]-aParis[1])+aParis[1]
r.zeta2LT=r.zeta2LT[Z.year>=1940]
a.zeta2LT=r.zeta2LT[!is.na(r.zeta2LT)]

l.cor=c()
nexp=100
for(i in 1:nexp){
RN=trednoise(a.zeta2LT,varB.TGparis,length(a.zeta2LT))
L.RN=lifetime2(RN)
corXY=cor(a.zeta2LT,L.RN,use="pairwise.complete")
l.cor=c(l.cor,corXY)
}
boxplot(l.cor)

